#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>

#include <gtk/gtk.h>

#include "defines.h"
#include "../config.h"

#include <gksu.h>
#include <gksutil.h>

struct option long_opts[] = {
    /*
     * { name  has_arg  *flag  val } 
     */
    {"help", no_argument, NULL, 'h'},
    {"login", no_argument, NULL, 'l'},
    {"preserv-env", no_argument, NULL, 'k'},
    {"user", required_argument, NULL, 'u'},
    {"print-pass", no_argument, NULL, 'p'},
    {"message", required_argument, NULL, 'm'},
    {"title", required_argument, NULL, 't'},
    {"icon", required_argument, NULL, 'i'},
    {"disable-grab", no_argument, NULL, 'g'},
    {"ssh-fwd", no_argument, NULL, 's'},
    {"debug", no_argument, NULL, 'd'},
    {0, 0, 0, 0}
};

/**
 * help:
 * @cmdname:  name of the command which was called by the user 
 * (argv[0])
 *
 * This function is a simple 'usage'-style printing function.
 * It is called if the user calls the program with --help or -h
 */
void
help (gchar *cmdname)
{ 
  g_print (_(
"GKsu version %s\n\n"
"Usage: %s [-u <user>] [-k] [-l] <command>\n\n"
"  --user <user>, -u <user>\n"
"    Calls <command> as the specified user\n"
"  --message <message>, -m <message>\n"
"    Replaces the standard message shown to ask for\n"
"    password for the argument passed to the option\n"
"  --title <title>, -t <title>\n"
"    Replaces the default title with the argument\n"
"  --icon <icon>, -i <icon>\n"
"    Replaces the default window icon with the argument\n"
"  --print-pass, -p\n"
"    Asks gksu to print the password to stdout, just\n"
"    like ssh-askpass. Useful to use in scripts with\n"
"    programs that accept receiving the passowrd on\n"
"    stdin.\n"
"  --disable-grab, -g\n"
"    Disables the \"locking\" of the keyboard, mouse,\n"
"    and focus done by the program when asking for\n"
"    password\n"
"  --ssh-fwd, -s\n"
"    Strip the host part of the $DISPLAY variable, so that\n"
"    GKSu will work on SSH X11 Forwarding.\n"
"\n" 
"  --login, -l\n"
"    Makes this a login shell. Beware this may cause\n"
"    problems with the Xauthority magic. Run xhost\n"
"    to allow the target user to open windows on your\n"
"    display!\n"
"  --preserv-env, -k\n"
"    Preserve the current environments, does not set $HOME\n"
"    nor $PATH, for example.\n"
"\n"
"\n"), PACKAGE_VERSION, cmdname);
}

int
main (int argc, char **argv)
{
  gboolean print_pass = FALSE;
  gchar *command;
  gchar *password;
  gboolean sudo_mode = FALSE;
  int c;

  setlocale (LC_ALL, "");
  bindtextdomain(PACKAGE_NAME, LOCALEDIR);  
  bind_textdomain_codeset (PACKAGE_NAME, "UTF-8");
  textdomain(PACKAGE_NAME);

  { /* support gksu_sudo_run */
    gchar *myname = g_path_get_basename (argv[0]);
    if (!strcmp(myname, "gksudo"))
      sudo_mode = TRUE;
    g_free (myname);
  }

  if (gksu_init (argc, argv))
    return 1;

  while ((c = getopt_long(argc, argv, "?hu:lpm:kt:i:gds", long_opts, NULL))
	 != EOF)
    {
      switch (c)
	{
	case 'h':
	  help (argv[0]);
	  exit(0);
	  break;
	case '?':
	  help (argv[0]);
	  exit(0);
	  break;
	case 'u':
	  gksu_set_user (optarg);
	  break;
	case 'l':
	  gksu_set_login_shell (TRUE);
	  break;
	case 'p':
	  print_pass = TRUE;
	  break;
	case 't':
	  gksu_set_title (optarg);
	  break;
	case 'm':
	  gksu_set_message (optarg);
	  break;
	case 'i':
	  gksu_set_icon (optarg);
	  break;
	case 'k':
	  gksu_set_keep_env (TRUE);
	  break;
	case 'g':
	  gksu_set_grab (FALSE);
	  break;
	case 'd':
	  gksu_set_debug (TRUE);
	  break;
	case 's':
	  gksu_set_ssh_fwd (TRUE);
	  break;
	default:
	  fprintf (stderr, _("Using %s as username...\n"), argv[optind]);
	}
    }

  if (print_pass)
    {
      if (!gksu_get_message ())
	{
	  gchar *msg = 
	    g_strdup_printf (_("Please enter %s's password"),
			     gksu_get_user ());

	  gksu_set_message (msg);
	  g_free (msg);
	}

      password = gksu_ask_password ();
      printf ("%s\n", password);
      return 0;
    }

  /* now we can begin to care about a command */
  if (argc == optind)
    {
      gk_dialog (GTK_MESSAGE_ERROR, _("Missing command to run."));
      return 1;
    }
  command = g_strdup (argv[optind]);
  if (sudo_mode)
    gksu_sudo_run (command);
  else
    gksu_run (command);

  return 0;
}
