/*
 * GkSu -- Gtk+ Frontend to su
 * Copyright (C) 2002 Gustavo Noronha Silva
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <libgen.h> /* for basename() */
#include <fcntl.h>
#include <pty.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/select.h>
#include <errno.h>

#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <X11/Xlib.h>
#include <locale.h>

#include <gksutil.h>

#include "defines.h"
#include "../config.h"

#include "gksu-private.h"
#include "gksu.h"

/* no need to return / pass around */
GkConf *_gksu_conf;

/**
 * gksu_secure_free:
 * @string: string to be freed securely
 *
 * Clears variables filling them with 0's previously
 */
void
gksu_secure_free (gchar *string)
{
  if (string != NULL)
    {
      memset (string, 0, strlen(string));
      g_free (string);
    }
}

/**
 * prepare_xauth:
 *
 * Sets up the variables with values for the $DISPLAY 
 * environment variable and xauth-related stuff. Also
 * creates a temporary directory to hold a .Xauthority
 *
 * Returns: 0 if it suceeds.
 */
int
prepare_xauth ()
{
  FILE *xauth_output;
  gchar xauth[256] = {0};

  gchar *tmp = NULL;
  gchar **tmpv = NULL;
  gchar *display = NULL;

  if (_gksu_conf->ssh_fwd)
    {
      display = g_strdup (getenv ("DISPLAY"));
      tmpv = g_strsplit (display, ":", 3);
      g_free (display);

      _gksu_conf->display = g_strdup_printf (":%s", tmpv[1]);

      g_strfreev (tmpv);
    }
  else
    _gksu_conf->display = g_strdup (getenv ("DISPLAY"));

  /* get the authorization token */
  tmp = g_strdup_printf ("/usr/X11R6/bin/xauth list %s | "
			 "grep 'MIT-MAGIC-COOKIE-1' | "
			 "cut -d ' ' -f 5",
			 _gksu_conf->display);
  if ((xauth_output = popen (tmp, "r")) == NULL)
    {
      gk_dialog (GTK_MESSAGE_ERROR,
                 _("Failed to obtain xauth key: %s"),
                 strerror(errno));
      return 1;
    }
  fread (xauth, sizeof(char), 255, xauth_output);
  xauth[strlen(xauth) - 1] = '\0';
  pclose (xauth_output);
  g_free (tmp);

  _gksu_conf->xauth = g_strdup (xauth);

  if (_gksu_conf->debug)
    {
      fprintf(stderr, 
	      "xauth: -%s-\n"
	      "display: -%s-\n",
	      _gksu_conf->xauth, _gksu_conf->display);
    }

  return 0;
}

gboolean
sudo_prepare_xauth ()
{
  gchar template[] = "/tmp/" PACKAGE "-XXXXXX";
  gchar *xauth;

  _gksu_conf->dir = g_strdup (mkdtemp(template));
  if (!_gksu_conf->dir)
    {
      gk_dialog (GTK_MESSAGE_ERROR, strerror(errno));
      return 1;
    }
  
  xauth = g_strdup_printf("%s/.Xauthority", g_get_home_dir());
  copy(xauth, _gksu_conf->dir);
  g_free (xauth);

  return 0;
}

/**
 * gksu_init:
 * @argc: argc to be passed to gtk_init, mainly - or zero
 * @argv: argv to be passed to gtk_init, or NULL
 *
 * Inits the main configuration struct, a #GkConf, and also
 * inits the Gtk+ engine. You need to call this before doing
 * anything.
 *
 * Returns: 0 if it suceeds, 1 if it fails.
 */
gboolean 
gksu_init (gint argc, gchar **argv)
{
  _gksu_conf = g_new (GkConf, 1);

  _gksu_conf->xauth = NULL;
  _gksu_conf->dir = NULL;
  _gksu_conf->display = NULL;
  _gksu_conf->user = NULL;
  _gksu_conf->command = NULL; /* gksu */

  _gksu_conf->title = NULL;
  _gksu_conf->message = NULL;
  _gksu_conf->icon = NULL;

  _gksu_conf->login_shell = FALSE;
  _gksu_conf->keep_env = FALSE;
  _gksu_conf->grab = TRUE;

  _gksu_conf->debug = FALSE;
  _gksu_conf->ssh_fwd = FALSE;

  _gksu_conf->user = g_strdup ("root");

  if (!gtk_init_check (&argc, &argv))
    {
      fprintf (stderr, _("GKSu: Failed to initialize the Gtk+ library. "
			 "GKSu will not be able to continue!\n"));
      return 1;
    }

  return 0;
}

void
gksu_free_conf ()
{
  g_free (_gksu_conf->xauth);
  g_free (_gksu_conf->dir);
  g_free (_gksu_conf->display);
  g_free (_gksu_conf->user);
  g_free (_gksu_conf->command);

  g_free (_gksu_conf->title);
  g_free (_gksu_conf->message);
  g_free (_gksu_conf->icon);

  g_free (_gksu_conf->user);

  g_free (_gksu_conf);
}

/**
 * gksu_ask_password:
 *
 * This is one of the main functions in gksu. It asks the
 * user for the password of the target user.
 *
 * Returns: a newly allocated gchar containing the password
 * or NULL if an error happens or the user cancels the action
 */
gchar*
gksu_ask_password ()
{
  gchar *pass = NULL;
	
  GtkWidget
    *dialog, 
    *hbox,
    *vbox,
    *image,
    *entry, 
    *label;

  gint response;

  if (_gksu_conf->title == NULL)
    _gksu_conf->title = g_strdup (_("Changing user..."));

  if (_gksu_conf->icon == NULL)
    gtk_window_set_default_icon_from_file (DATA_DIR "/pixmaps/gksu.png", 
					   NULL);
  else
    gtk_window_set_default_icon_from_file (_gksu_conf->icon, NULL);

  /* dialog window */
  dialog = gtk_dialog_new_with_buttons (_gksu_conf->title, NULL, GTK_DIALOG_MODAL,
					GTK_STOCK_CANCEL,
					GTK_RESPONSE_CANCEL,
					GTK_STOCK_OK,
					GTK_RESPONSE_OK,
					NULL);

  gtk_dialog_set_has_separator (GTK_DIALOG(dialog), FALSE);
  gtk_container_set_border_width (GTK_CONTAINER(dialog), 6);
  gtk_box_set_spacing (GTK_BOX(GTK_DIALOG(dialog)->vbox), 12);
  /* Center window and prepare for grab */
  gtk_window_set_default_size (GTK_WINDOW(dialog), 250, 100);
  gtk_window_set_position (GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
  gtk_window_stick (GTK_WINDOW(dialog));

  /* hbox */
  hbox = gtk_hbox_new (FALSE, 12);
  gtk_container_set_border_width (GTK_CONTAINER(hbox), 6);
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox),
		      hbox, TRUE, TRUE, 0);
  gtk_widget_show (hbox);

  /* image */
  if (_gksu_conf->icon == NULL)
    image = gtk_image_new_from_file (DATA_DIR"/pixmaps/gksu-auth.png");
  else
    image = gtk_image_new_from_file (_gksu_conf->icon);
  gtk_misc_set_alignment (GTK_MISC(image), 0.5, 0);
  gtk_box_pack_start (GTK_BOX(hbox), image, FALSE, FALSE, 0);
  gtk_widget_show (image);

  /* vbox for label and entry */
  vbox = gtk_vbox_new (FALSE, 12);
  gtk_box_pack_start (GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
  gtk_widget_show (vbox);

  /* label */
  label = gtk_label_new (_gksu_conf->message);
  gtk_label_set_justify (GTK_LABEL(label), GTK_JUSTIFY_CENTER);
  gtk_label_set_use_markup (GTK_LABEL(label), TRUE);
  gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0);
  gtk_box_pack_start (GTK_BOX(vbox), label, TRUE, TRUE, 0);
  gtk_widget_show (label);

  /* hbox for entry and label */
  hbox = gtk_hbox_new (FALSE, 6);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
  gtk_widget_show (hbox);

  /* entry label */
  label = gtk_label_new (_("Password:"));
  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);
  gtk_widget_show (label);
  
  /* entry */
  entry = gtk_entry_new();
  gtk_signal_connect (GTK_OBJECT(entry), "activate",
		      GTK_SIGNAL_FUNC(ask_pass_ok_cb),
		      dialog);
  gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
  gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
  gtk_widget_show (entry);
  
  gtk_widget_show_all (dialog);
  gtk_widget_grab_focus(entry);

  while (gtk_events_pending())
    gtk_main_iteration ();

  if (_gksu_conf->grab)
    {
      struct timeval tv;

      tv.tv_sec = 0;
      tv.tv_usec = 2000;

      /* Grab focus */
      XGrabServer(GDK_DISPLAY());

      /* wait a bit longer for the grab, 
	 just in case system load is high */      
      select (0, NULL, NULL, NULL, &tv); 

      if (gdk_pointer_grab(dialog->window, TRUE, 0, NULL, NULL, GDK_CURRENT_TIME) ||
	  gdk_keyboard_grab(dialog->window, FALSE, GDK_CURRENT_TIME))
	{
	  gdk_pointer_ungrab(GDK_CURRENT_TIME);
	  XUngrabServer(GDK_DISPLAY());
	  gtk_widget_hide (dialog);

	  while (gtk_events_pending())
	    gtk_main_iteration ();

	  report_failed_grab ();
	  gtk_widget_show (dialog);
	}
    }

  /* let the magic begin */
  response = gtk_dialog_run (GTK_DIALOG(dialog));
  
  if (_gksu_conf->grab)
    {
      /* Ungrab */
      XUngrabServer(GDK_DISPLAY());
      gdk_pointer_ungrab(GDK_CURRENT_TIME);
      gdk_keyboard_ungrab(GDK_CURRENT_TIME);
      gdk_flush();
    }

  /* Report passphrase if user selected OK */
  pass = g_strdup (gtk_entry_get_text(GTK_ENTRY(entry)));
		
  /* Kill the entry's copy of the passphrase. */
  gtk_entry_set_text(GTK_ENTRY(entry), "");
			
  switch (response)
    {
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_NONE:
      gksu_secure_free (pass);
      return 0;
    }

  gtk_widget_hide (dialog);

  while (gtk_events_pending())
    gtk_main_iteration();

  return pass;
}

/**
 * ask_pass_ok_cb:
 * @w: activated widget
 * @data: the #GtkDialog to which a response should be emitted.
 *
 * This function handles cases in which the default widget
 * of the ask_pass dialog is not the one activated to 'OK'
 * the dialog. This function will, then, emmit the GTK_RESPONSE_OK
 * to the dialog passed on @data.
 */
void
ask_pass_ok_cb (GtkWidget *w, gpointer data)
{
  GtkWidget *dialog = (GtkWidget*)data;

  gtk_dialog_response (GTK_DIALOG(dialog),
		       GTK_RESPONSE_OK);
}

/**
 * report_failed_grab:
 *
 * Simple utility function called to report a fail on
 * the process of grabbing the mouse or keyboard when
 * asking for the password through ask_password()
 */
void
report_failed_grab (void)
{
  gk_dialog (GTK_MESSAGE_WARNING, 
	     _("Could not grab keyboard or mouse.\n"
	       "A malicious client may be eavesdropping\n"
	       "on your session."));
}

/**
 * gksu_run:
 * @command: string containing the command to be run
 *
 * This could be considered one of the main functions in GKSu.
 * it is responsible for doing the 'user changing' magic by
 * calling gksu_ask_password() if it needs the target user's 
 * password.
 *
 * Returns: TRUE(1) if failed, FALSE(0) if succeeded.
 */
gboolean
gksu_run (gchar *command)
{
  int i = 0;
  gboolean auth_failed = FALSE;

  int fdpty;
  pid_t pid;

  gchar *password = NULL;
  gchar *pass = NULL;

  /* use the locally passed command */
  if (_gksu_conf->command && command)
    g_free (_gksu_conf->command);
  _gksu_conf->command = command;

  /* ask for password */
  if (_gksu_conf->message == NULL)
    _gksu_conf->message = 
      g_strdup_printf (_("<b>I need %s's password to run:</b>\n"
			 "<i>%s</i>"), 
		       _gksu_conf->user, _gksu_conf->command);
  else
    _gksu_conf->message = g_strdup (_gksu_conf->message);

  /* have X authorization stuff dealt with */
  prepare_xauth ();

  pid = forkpty (&fdpty, NULL, NULL, NULL);
  if (pid == 0)
    {
      gchar **cmd = g_malloc (sizeof(gchar*)*7);

      cmd[i] = g_strdup ("/bin/su"); i++;
      if (_gksu_conf->login_shell)
	{
	  cmd[i] = g_strdup ("-"); i++;
	}
      cmd[i] = g_strdup (_gksu_conf->user); i++;
      if (_gksu_conf->keep_env)
	{
	  cmd[i] = g_strdup ("-p"); i++;
	}
      cmd[i] = g_strdup ("-c"); i++;

      /* needs to get X authorization prior to running the program */
      cmd[i] = g_strdup_printf (PREFIX "/lib/gksu/gksu-run-helper \"%s\"",
				_gksu_conf->command); i++;

      cmd[i] = NULL;

      /* executes the command */
      if (execv (cmd[0], cmd) == -1)
	{
	  gk_dialog (GTK_MESSAGE_ERROR, 
		     _("Unable to run /bin/su: %s"),
		     strerror(errno));
	}

      for (i = 0 ; cmd[i] != NULL ; i++)
	g_free (cmd[i]);
      g_free(cmd);
    }
  else if (pid == -1)
    {
      gk_dialog (GTK_MESSAGE_ERROR, 
		 _("Failed to fork new process: %s"),
		 strerror(errno));
      return 1;
    }
  else
    {
      fd_set rfds;

      struct timeval tv;

      struct passwd *pwd = NULL;
      gint target_uid = -1;
      gint my_uid = 0;

      gchar buf[256];
      gint status;

      /* 
	 for controling the conversation with the
	 helper
      */
      gboolean first_line_read = FALSE;

      read (fdpty, buf, 256);
      
      my_uid = getuid();
      pwd = getpwnam (_gksu_conf->user);
      if (pwd)
	target_uid = pwd->pw_uid;

      /* no need to ask for password if we're already root */
      if (my_uid != target_uid && my_uid)
	{
	  password = gksu_ask_password (_gksu_conf);

	  if (password == NULL)
	    {
	      write (fdpty, "\n", 1);
	      return 1;
	    }

	  /* we cannot pass 'password' as it has no \n */
	  pass = g_strdup_printf ("%s\n", password);
	  write (fdpty, pass, strlen(pass));

	  /* cleans the memory */
	  gksu_secure_free (pass);
	  gksu_secure_free (password);
	} 

      FD_ZERO(&rfds);
      FD_SET(fdpty, &rfds);

      while (!waitpid (pid, &status, WNOHANG))
	{
	  tv.tv_sec = 0;
	  tv.tv_usec = 100;
	  if(select (fdpty+1, &rfds, NULL, NULL, &tv) < 0)
	    {
	      gk_dialog (GTK_MESSAGE_ERROR, 
			 strerror(errno));
	      return 1;
	    }

	  if(FD_ISSET(fdpty, &rfds))
	    {
	      bzero(buf, 256);
	      read (fdpty, buf, 255);

	      if (!first_line_read)
		{
		  if (!strncmp (buf, "su: Authentication failure", 26))
		    auth_failed = TRUE;
		  else if (!strncmp (buf, "gksu: waiting", 13))
		    {
		      gchar *line;
		      
		      line = g_strdup_printf ("gksu-run: %s\n", _gksu_conf->display);
		      write (fdpty, line, strlen(line));
		      g_free (line);
		      
		      line = g_strdup_printf ("gksu-run: %s\n", _gksu_conf->xauth);
		      write (fdpty, line, strlen(line));
		      g_free (line);

		      first_line_read = TRUE;
		      bzero (buf, 255);
		    }
		}
	      else if (!strncmp (buf, "gksu-run:", 8))
		{
		  gchar *tmp = NULL;

		  tmp = g_strrstr (buf, "\n");
		  while (!tmp)
		    {
		      bzero(buf, 256);
		      read (fdpty, buf, 255);
		      tmp = g_strrstr (tmp, "\n");
		    }
		  if (tmp)
		    fprintf (stderr, "%s", tmp+1);
		  bzero (buf, 255);
		}
	      else
		fprintf (stderr, "%s", buf);
	    }
	  else
	    FD_SET(fdpty, &rfds);
	}

      while (1)
	{
	  bzero(buf, 256);
	  if(read (fdpty, buf, 255) <= 0)
	    break;
	  fprintf (stderr, "%s", buf);
	}

      if (WIFEXITED(status))
	{
	  if (WEXITSTATUS(status))
	    {
	      if (auth_failed)
		gk_dialog (GTK_MESSAGE_ERROR,
			   _("Wrong password."));
	      else
		gk_dialog (GTK_MESSAGE_INFO,
			   _("Child terminated with %d status"),
			   WEXITSTATUS(status));
	      return 1;
	    }
	}
    }

  return 0;
}

/**
 * gksu_sudo_run:
 * @command: string containing the command to be run
 *
 * This could be considered one of the main functions in GKSu.
 * it is responsible for doing the 'user changing' magic by
 * calling gksu_ask_password() if it needs the user's password
 * it behaves like sudo.
 *
 * Returns: TRUE(1) if failed, FALSE(0) if succeeded.
 */
gboolean
gksu_sudo_run (gchar *command)
{
  char **cmd;
  char buffer[256];
  int argcount = 8;
  int i, j;

  gchar *xauth, *xauth_env;

  pid_t pid;
  int status;
  size_t r;
  char buf[16];
  FILE *infile, *outfile;
  int parent_pipe[2];	/* For talking to the parent */
  int child_pipe[2];	/* For talking to the child */
  gchar *password;
  
  /* use the locally passed command */
  if (_gksu_conf->command && command)
    g_free (_gksu_conf->command);
  _gksu_conf->command = command;

  /* ask for password */
  if (_gksu_conf->message == NULL)
    _gksu_conf->message = 
      g_strdup_printf (_("<b>I need your password to run:</b>\n"
			 "<i>%s</i>"), _gksu_conf->command);
  else
    _gksu_conf->message = g_strdup (_gksu_conf->message);

  sudo_prepare_xauth ();

  /* sets XAUTHORITY */
  xauth = g_strdup_printf ("%s/.Xauthority", _gksu_conf->dir);
  xauth_env = getenv ("XAUTHORITY");
  setenv ("XAUTHORITY", xauth, TRUE);

  if (_gksu_conf->debug)
    fprintf (stderr, "xauth: %s\n", xauth);

  void
    reset_xauth ()
    {
      /* reset the env var as it was before or clean it  */
      if (xauth_env)
	setenv ("XAUTHORITY", xauth_env, TRUE);
      else
	unsetenv ("XAUTHORITY");

      unlink (xauth);
      rmdir (_gksu_conf->dir);
      g_free (xauth);
    }

  cmd = g_new (gchar *, argcount + 1);
  
  cmd[0] = g_strdup("/usr/bin/sudo"); /* sudo binary */
  cmd[1] = g_strdup("-H"); /* Make sudo set $HOME */
  cmd[2] = g_strdup("-S"); /* Make sudo read from stdin */
  cmd[3] = g_strdup("-p"); /* Make sudo use next arg as prompt */
  cmd[4] = g_strdup("GNOME_SUDO_PASS ");	/* prompt */
  cmd[5] = g_strdup("-u"); /* Make sudo use the selected user */
  cmd[6] = g_strdup(_gksu_conf->user);    /* user */
  cmd[7] = g_strdup("--"); /* Make sudo stop processing options */  

  for (i = j = 0; ; i++)
    {
      if (command[i] == ' ' || command[i] == '\0')
	{
	  buffer[j] = '\0';
	  cmd = g_realloc (cmd, sizeof(gchar*) * (argcount + 1));
	  cmd[argcount] = g_strdup (buffer);
	  bzero (buffer, 256);
	  argcount = argcount + 1;
	  j = 0;

	  if (command[i] == '\0')
	    break;
	}
      else
	{
	  if (command[i] == '\\')
	    i = i + 1;
	  buffer[j] = command[i];
	  j = j + 1;
	}
    }
  cmd = g_realloc (cmd, sizeof(gchar*) * (argcount + 1));
  cmd[argcount] = NULL;

  if (_gksu_conf->debug)
    {
      for (i = 0; cmd[i] != NULL; i++)
	fprintf (stderr, "cmd[%d]: %s\n", i, cmd[i]);
    }

  if ((pipe(parent_pipe)) == -1)
    {
      gk_dialog (GTK_MESSAGE_ERROR, 
		 _("Error creating pipe: %s"),
		 strerror(errno));
      reset_xauth ();
      return 1;
    }

  if ((pipe(child_pipe)) == -1)
    {
      gk_dialog (GTK_MESSAGE_ERROR, 
		 _("Error creating pipe: %s"),
		 strerror(errno));
      reset_xauth ();
      return 1;
    }

  pid = fork();
  if (pid == -1)
    {
      gk_dialog (GTK_MESSAGE_ERROR, 
		 _("Failed to fork new process: %s"),
		 strerror(errno));
      reset_xauth ();
      return 1;
    }
  else if (pid == 0) 
    {
      // Child
      close(child_pipe[1]);
      dup2(child_pipe[0], STDIN_FILENO);
      dup2(parent_pipe[1], STDERR_FILENO);
      
      execv(cmd[0], cmd);
      
      gk_dialog (GTK_MESSAGE_ERROR, 
		 _("Failed to exec new process: %s"),
		 strerror(errno));
      reset_xauth ();
      return 1;
    } 
  else 
    {
      // Parent
      close(parent_pipe[1]);
      
      /*
	we are expecting to receive a GNOME_SUDO_PASS
	if we don't there are two possibilities: an error
	or a password is not needed
      */
      r = read(parent_pipe[0], buf, 16);

      infile = fdopen(parent_pipe[0], "r");
      if (!infile)
	{
	  reset_xauth ();
	  return 1;
	}

      outfile = fdopen(child_pipe[1], "w");
      if (!outfile)
	{
	  reset_xauth ();
	  return 1;
	}

      if (strncmp(buf, "GNOME_SUDO_PASS ", 16) == 0) 
	{
	  password = gksu_ask_password ();
	  if (!password)
	    {
	      fprintf(outfile, "\n");
	      reset_xauth ();
	      return 1;
	    }
	  
	  fprintf(outfile, "%s\n", password);
	  fclose(outfile);
	  
	  /* Zero passphrase in memory */
	  gksu_secure_free (password);
	} 
      else
	fprintf (stderr, "%s", buf);

      while (!waitpid (pid, &status, WNOHANG))
	{
	  bzero(buf, 256);
	  if(!fread (buf, sizeof(gchar), 255, infile))
	    break;
	  fprintf (stderr, "%s", buf);
	}

      /* make sure we did read everything */
      while (1)
	{
	  bzero(buf, 256);
	  if(!fread (buf, sizeof(gchar), 255, infile))
	    break;
	  fprintf (stderr, "%s", buf);
	  fflush (stderr);
	}
      
      if (WIFEXITED(status))
	{
	  if (WEXITSTATUS(status))
	    {
	      gk_dialog (GTK_MESSAGE_INFO,
			 _("Child terminated with %d status"),
			 WEXITSTATUS(status));
	      reset_xauth ();
	      return 1;
	    }
	}
      
      fclose(infile);
    }
  
  reset_xauth ();
  return 0;
}

/**
 * gksu_get_conf:
 *
 * Gets the #GkConf structure used internally to store
 * the configuration for direct manipulation.
 *
 * Returns: the #GkConf that is being used.
 */
GkConf*
gksu_get_conf ()
{
  return _gksu_conf;
}

/**
 * gksu_set_user:
 * @username: the target username
 *
 * Sets up what user the command will be run as. The default
 * is root, but you can run the command as any user.
 *
 */
void
gksu_set_user (gchar *username)
{
  g_assert (username != NULL);

  if (_gksu_conf->user)
    g_free (_gksu_conf->user);
  _gksu_conf->user = g_strdup (username);
}

/**
 * gksu_get_user:
 *
 * Gets the user the command will be run as, as set 
 * by gksu_set_user.
 *
 * Returns: a pointer to the string containing the username.
 */
const gchar*
gksu_get_user (void)
{
  return _gksu_conf->user;
}

void
gksu_set_login_shell (gboolean value)
{
  _gksu_conf->login_shell = value;
}

/**
 * gksu_get_login_shell:
 *
 * Finds out if the shell created by the underlying su process
 * will be a login shell.
 *
 * Returns: TRUE if the shell will be a login shell, FALSE otherwise.
 */
gboolean
gksu_get_login_shell ()
{
  return _gksu_conf->login_shell;
}

void
gksu_set_title (gchar *title)
{
  g_assert (title != NULL);

  if (_gksu_conf->title)
    g_free (_gksu_conf->title);
  _gksu_conf->title = conv_to_utf8 (title);
}

/**
 * gksu_get_title:
 *
 * Gets what is the current title that the windows that
 * asks for the password will show.
 *
 * Returns: a string with the current title
 */
const gchar*
gksu_get_title (void)
{
  return _gksu_conf->title;
}

void
gksu_set_message (gchar *message)
{
  g_assert (message != NULL);

  if (_gksu_conf->message)
    g_free (_gksu_conf->message);
  _gksu_conf->message = conv_to_utf8 (message);
}

const gchar *
gksu_get_message ()
{
  return _gksu_conf->message;
}

void
gksu_set_icon (gchar *icon)
{
  g_assert (icon != NULL);

  if (_gksu_conf->icon)
    g_free (_gksu_conf->icon);
  _gksu_conf->icon = g_strdup (icon);
}

const gchar*
gksu_get_icon (void)
{
  return _gksu_conf->icon;
}

void
gksu_set_keep_env (gboolean value)
{
  _gksu_conf->keep_env = value;
}

gboolean
gksu_get_keep_env ()
{
  return _gksu_conf->keep_env;
}

void
gksu_set_grab (gboolean value)
{
  _gksu_conf->grab = value;
}

gboolean
gksu_get_grab ()
{
  return _gksu_conf->grab;
}

void
gksu_set_debug (gboolean value)
{
  _gksu_conf->debug = value;
}

gboolean
gksu_get_debug ()
{
  return _gksu_conf->debug;
}

void
gksu_set_ssh_fwd (gboolean value)
{
  _gksu_conf->ssh_fwd = value;
}

gboolean
gksu_get_ssh_fwd ()
{
  return _gksu_conf->ssh_fwd;
}
